import unittest

from tira.tira_redirects import redirects, mirror_url
import requests
from hashlib import md5


def md5_of_first_kilobyte_of_http_resource(url):
    if not url.startswith('https://files.webis.de') and not url.startswith('https://zenodo.org/records'):
        raise ValueError(f'URL {url} is not from webis.de respectively zenodo')
    return md5(requests.get(url, headers={'Range': f'bytes=0-1024'}).content).hexdigest()

class TestRedirects(unittest.TestCase):
    def test_redirect_ir_lab_leipzig_wise23_train_inputs(self):
        expected_redirect = 'https://zenodo.org/records/10628882/files/training-20231104-inputs.zip?download=1'
        dataset_url = 'https://www.tira.io/data-download/training/input-/training-20231104-training.zip'

        expected_md5_first_kilobyte = '6e2e445a72e282c25c545154b427d3fd'

        actual_redirect =  redirects(url=dataset_url)['urls'][0]
        actual_mirror = mirror_url(actual_redirect)
        actual_md5_first_kilobyte = md5_of_first_kilobyte_of_http_resource(actual_redirect)


        self.assertEqual(expected_redirect, actual_redirect)
        self.assertEqual(expected_redirect, actual_mirror)
        self.assertEqual(expected_md5_first_kilobyte, actual_md5_first_kilobyte)

    def test_redirect_ir_lab_leipzig_wise23_train_truths(self):
        expected_redirect = 'https://zenodo.org/records/10628882/files/training-20231104-truth.zip?download=1'
        dataset_url = 'https://www.tira.io/data-download/training/input-truth/training-20231104-training.zip'
        expected_md5_first_kilobyte = '6d1ef9e09c40e9b0664366a2a003fa64'

        actual_redirect =  redirects(url=dataset_url)['urls'][0]
        actual_mirror = mirror_url(actual_redirect)
        actual_md5_first_kilobyte = md5_of_first_kilobyte_of_http_resource(actual_redirect)

        self.assertEqual(expected_redirect, actual_redirect)
        self.assertEqual(expected_redirect, actual_mirror)
        self.assertEqual(expected_md5_first_kilobyte, actual_md5_first_kilobyte)

    def test_redirect_ir_lab_padua_sose23_train_inputs(self):
        expected_redirect = 'https://zenodo.org/records/10628882/files/training-20231104-inputs.zip?download=1'
        dataset_url = 'https://www.tira.io/data-download/training/input-/longeval-tiny-train-20240315-training.zip'

        expected_md5_first_kilobyte = '6e2e445a72e282c25c545154b427d3fd'

        actual_redirect =  redirects(url=dataset_url)['urls'][0]
        actual_mirror = mirror_url(actual_redirect)
        actual_md5_first_kilobyte = md5_of_first_kilobyte_of_http_resource(actual_redirect)


        self.assertEqual(expected_redirect, actual_redirect)
        self.assertEqual(expected_redirect, actual_mirror)
        self.assertEqual(expected_md5_first_kilobyte, actual_md5_first_kilobyte)

    def test_redirect_ir_lab_padua_sose23_train_truths(self):
        expected_redirect = 'https://zenodo.org/records/10628882/files/training-20231104-truth.zip?download=1'
        dataset_url = 'https://www.tira.io/data-download/training/input-truth/longeval-tiny-train-20240315-training.zip'
        expected_md5_first_kilobyte = '6d1ef9e09c40e9b0664366a2a003fa64'

        actual_redirect =  redirects(url=dataset_url)['urls'][0]
        actual_mirror = mirror_url(actual_redirect)
        actual_md5_first_kilobyte = md5_of_first_kilobyte_of_http_resource(actual_redirect)

        self.assertEqual(expected_redirect, actual_redirect)
        self.assertEqual(expected_redirect, actual_mirror)
        self.assertEqual(expected_md5_first_kilobyte, actual_md5_first_kilobyte)

    def test_redirect_ir_lab_leipzig_wise23_jena_inputs(self):
        expected_redirect = 'https://zenodo.org/records/10628882/files/jena-topics-small-20240119-inputs.zip?download=1'
        dataset_url = 'https://www.tira.io/data-download/training/input-/jena-topics-small-20240119-training.zip'

        expected_md5_first_kilobyte = 'b013dcb175648f0190fb5b5ab9f8cc5b'

        actual_redirect =  redirects(url=dataset_url)['urls'][0]
        actual_mirror = mirror_url(actual_redirect)
        actual_md5_first_kilobyte = md5_of_first_kilobyte_of_http_resource(actual_redirect)


        self.assertEqual(expected_redirect, actual_redirect)
        self.assertEqual(expected_redirect, actual_mirror)
        self.assertEqual(expected_md5_first_kilobyte, actual_md5_first_kilobyte)

    def test_redirect_ir_lab_leipzig_wise23_jena_truths(self):
        expected_redirect = 'https://zenodo.org/records/10628882/files/jena-topics-small-20240119-truth.zip?download=1'
        dataset_url = 'https://www.tira.io/data-download/training/input-truth/jena-topics-small-20240119-training.zip'
        expected_md5_first_kilobyte = 'aa25f6931bed856fdb29d0374ff107e3'

        actual_redirect =  redirects(url=dataset_url)['urls'][0]
        actual_mirror = mirror_url(actual_redirect)
        actual_md5_first_kilobyte = md5_of_first_kilobyte_of_http_resource(actual_redirect)

        self.assertEqual(expected_redirect, actual_redirect)
        self.assertEqual(expected_redirect, actual_mirror)
        self.assertEqual(expected_md5_first_kilobyte, actual_md5_first_kilobyte)

    def test_redirect_ir_lab_leipzig_wise23_validation_inputs(self):
        expected_redirect = 'https://zenodo.org/records/10628882/files/validation-20231104-inputs.zip?download=1'
        expected_md5_first_kilobyte = '1ad750ebde6828971ec63cda1d415be6'
        dataset_url = 'https://www.tira.io/data-download/training/input-/validation-20231104-training.zip'

        actual_redirect =  redirects(url=dataset_url)['urls'][0]
        actual_mirror = mirror_url(actual_redirect)
        actual_md5_first_kilobyte = md5_of_first_kilobyte_of_http_resource(actual_redirect)

        self.assertEqual(expected_redirect, actual_redirect)
        self.assertEqual(expected_redirect, actual_mirror)
        self.assertEqual(expected_md5_first_kilobyte, actual_md5_first_kilobyte)

    def test_redirect_ir_lab_leipzig_wise23_leipzig_inputs(self):
        expected_redirect = 'https://zenodo.org/records/10628882/files/leipzig-topics-small-20240119-inputs.zip?download=1'
        dataset_url = 'https://www.tira.io/data-download/training/input-/leipzig-topics-small-20240119-training.zip'

        expected_md5_first_kilobyte = '09a02fdfb03c9797842ff7adc453f850'

        actual_redirect =  redirects(url=dataset_url)['urls'][0]
        actual_mirror = mirror_url(actual_redirect)
        actual_md5_first_kilobyte = md5_of_first_kilobyte_of_http_resource(actual_redirect)


        self.assertEqual(expected_redirect, actual_redirect)
        self.assertEqual(expected_redirect, actual_mirror)
        self.assertEqual(expected_md5_first_kilobyte, actual_md5_first_kilobyte)

    def test_redirect_ir_lab_leipzig_wise23_leipzig_truths(self):
        expected_redirect = 'https://zenodo.org/records/10628882/files/leipzig-topics-small-20240119-truth.zip?download=1'
        dataset_url = 'https://www.tira.io/data-download/training/input-truth/leipzig-topics-small-20240119-training.zip'
        expected_md5_first_kilobyte = '7a60ab76ed8c3423bd407932a1e7e910'

        actual_redirect =  redirects(url=dataset_url)['urls'][0]
        actual_mirror = mirror_url(actual_redirect)
        actual_md5_first_kilobyte = md5_of_first_kilobyte_of_http_resource(actual_redirect)

        self.assertEqual(expected_redirect, actual_redirect)
        self.assertEqual(expected_redirect, actual_mirror)
        self.assertEqual(expected_md5_first_kilobyte, actual_md5_first_kilobyte)

    def test_redirect_ir_lab_leipzig_wise23_validation_truths(self):
        expected_redirect = 'https://zenodo.org/records/10628882/files/validation-20231104-truth.zip?download=1'
        expected_md5_first_kilobyte = '92ba3a87c1d4cceebccbefd1aa5f944e'
        dataset_url = 'https://www.tira.io/data-download/training/input-truth/validation-20231104-training.zip'

        actual_redirect =  redirects(url=dataset_url)['urls'][0]
        self.assertEqual(expected_redirect, actual_redirect)
        actual_mirror = mirror_url(actual_redirect)
        actual_md5_first_kilobyte = md5_of_first_kilobyte_of_http_resource(actual_redirect)


        self.assertEqual(expected_redirect, actual_redirect)
        self.assertEqual(expected_redirect, actual_mirror)
        self.assertEqual(expected_md5_first_kilobyte, actual_md5_first_kilobyte)

    def test_redirect_ir_lab_leipzig_sose23_inputs(self):
        expected_redirect = 'https://zenodo.org/records/10628640/files/iranthology-20230618-training-inputs.zip?download=1'
        expected_md5_first_kilobyte = 'f0727e93741e50b32270883e821075d0'
        dataset_url = 'https://www.tira.io/data-download/training/input-/iranthology-20230618-training.zip'

        actual_redirect =  redirects(url=dataset_url)['urls'][0]
        self.assertEqual(expected_redirect, actual_redirect)
        actual_mirror = mirror_url(actual_redirect)
        actual_md5_first_kilobyte = md5_of_first_kilobyte_of_http_resource(actual_redirect)

        self.assertEqual(expected_redirect, actual_mirror)
        self.assertEqual(expected_md5_first_kilobyte, actual_md5_first_kilobyte)

    def test_redirect_ir_lab_leipzig_sose23_truths(self):
        expected_redirect = 'https://zenodo.org/records/10628640/files/iranthology-20230618-training-truths.zip?download=1'
        expected_md5_first_kilobyte = '7dd9d2cb48c3bc8515fd0d5c973c5fd8'
        dataset_url = 'https://www.tira.io/data-download/training/input-truth/iranthology-20230618-training.zip'
        

        actual_redirect =  redirects(url=dataset_url)['urls'][0]
        actual_mirror = mirror_url(actual_redirect)
        actual_md5_first_kilobyte = md5_of_first_kilobyte_of_http_resource(actual_redirect)

        self.assertEqual(expected_redirect, actual_redirect)
        self.assertEqual(expected_redirect, actual_mirror)
        self.assertEqual(expected_md5_first_kilobyte, actual_md5_first_kilobyte)




#from approvaltests import verify_as_json
import unittest
from subprocess import check_output
from tira.tira_run import main
import sys
import tempfile
import os

def foo(image):
    orig_argv = list(sys.argv)
    temp_dir = '/tmp/' + tempfile.TemporaryDirectory().name
    os.makedirs(temp_dir, exist_ok=True)

    sys.argv = ['tira-run', '--input-dataset', 'workshop-on-open-web-search/document-processing-20231027-training', '--image', image, '--output-dir', 'fffff']
    main()
    sys.argv = orig_argv


class TiraRunIntegrationTest(unittest.TestCase):

    def testfoo(self):
        image = 'dockerfile_multi_stage_bash'
        check_output(['docker', 'build', '-t', image, '-f', f'tests/resources/{image}', '.'])
        
        #foo(image)
        #raise ValueError('sda')



from tira.local_client import Client

import pandas as pd
import unittest
from approvaltests import verify_as_json
import json


def retrieval_submission(queries):
    queries = pd.DataFrame([{'qid': str(i)} for i in queries])
    tira = Client('tests/resources/')
    retriever =  tira.pt.from_retriever_submission('ir-benchmarks/tira-ir-starters/retriever', dataset='d1')
    
    ret = retriever(queries)
    return [{'qid': i['qid'], 'docno': i['docno'], 'score': i['score']} for _, i in ret.iterrows()]

class TestPtFromRetrieverSubmissionLoaderTest(unittest.TestCase):
    def test_for_multiple_queries(self):
        expected = [
            {'qid': '1', 'docno': 'doc-1', 'score':10},
            {'qid': '1', 'docno': 'doc-2', 'score':9},
            {'qid': '3', 'docno': 'doc-3', 'score': 1},
        ]
        
        actual = retrieval_submission([1, 3])

        self.assertEqual(len(actual), 3)
        self.assertEqual(actual[0], expected[0])
        self.assertEqual(actual[1], expected[1])
        self.assertEqual(actual[2], expected[2])

    def test_cascading_with_retriever_is_idempotent_01(self):
        queries = pd.DataFrame([{'qid': str(i)} for i in [1]])
        tira = Client('tests/resources/')
        retriever =  tira.pt.from_retriever_submission('ir-benchmarks/tira-ir-starters/retriever', dataset='d1')
    
        ret = (retriever >> retriever).search(queries)
        ret = [json.loads((i.to_json())) for _, i in ret.iterrows()]
        verify_as_json(ret)

    def test_cascading_with_retriever_is_idempotent_02(self):
        queries = pd.DataFrame([{'qid': str(i)} for i in [1]])
        tira = Client('tests/resources/')
        retriever =  tira.pt.from_retriever_submission('ir-benchmarks/tira-ir-starters/retriever', dataset='d1')
    
        ret = (retriever >> retriever >> retriever >> retriever).search(queries)
        ret = [json.loads((i.to_json())) for _, i in ret.iterrows()]
        verify_as_json(ret)

    def test_cascading_with_retriever_is_idempotent_03(self):
        queries = pd.DataFrame([{'qid': str(i)} for i in [1]])
        tira = Client('tests/resources/')
        retriever =  tira.pt.from_retriever_submission('ir-benchmarks/tira-ir-starters/retriever', dataset='d1')
    
        ret = ((retriever >> retriever >> retriever >> retriever) >> (retriever >> retriever >> retriever >> retriever)).search(queries)
        ret = [json.loads((i.to_json())) for _, i in ret.iterrows()]
        verify_as_json(ret)

    def test_feature_union_from_retriever_01(self):
        queries = pd.DataFrame([{'qid': str(i)} for i in [1]])
        tira = Client('tests/resources/')
        retriever =  tira.pt.from_retriever_submission('ir-benchmarks/tira-ir-starters/retriever', dataset='d1')
    
        ret = (retriever >>(retriever ** retriever)).search(queries)
        ret = [json.loads((i.to_json())) for _, i in ret.iterrows()]
        verify_as_json(ret)


    def test_feature_union_from_retriever_02(self):
        queries = pd.DataFrame([{'qid': str(i)} for i in [1]])
        tira = Client('tests/resources/')
        retriever =  tira.pt.from_retriever_submission('ir-benchmarks/tira-ir-starters/retriever', dataset='d1')
    
        ret = (retriever >>(retriever ** retriever ** retriever ** retriever)).search(queries)
        ret = [json.loads((i.to_json())) for _, i in ret.iterrows()]
        verify_as_json(ret)

    def test_for_single_query_no_1(self):
        expected = [
            {'qid': '1', 'docno': 'doc-1', 'score':10},
            {'qid': '1', 'docno': 'doc-2', 'score':9},
        ]
        
        actual = retrieval_submission([1])

        self.assertEqual(len(actual), 2)
        self.assertEqual(actual[0], expected[0])
        self.assertEqual(actual[1], expected[1])

    def test_for_single_query_no_3(self):
        expected = [
            {'qid': '3', 'docno': 'doc-3', 'score': 1},
        ]

        actual = retrieval_submission([3])

        self.assertEqual(len(actual), 1)
        self.assertEqual(actual[0], expected[0])

    def test_retrieval_submission_from_rest_api(self):
        from tira.rest_api_client import Client
        from tira.third_party_integrations import ensure_pyterrier_is_loaded
        import pyterrier as pt
        ensure_pyterrier_is_loaded()
        tira = Client()

        q = tira.pt.from_submission('ir-benchmarks/tira-ir-starter/BM25 Re-Rank (tira-ir-starter-pyterrier)', pt.get_dataset("irds:disks45/nocr/trec-robust-2004"))
        self.assertEqual(len(q(pd.DataFrame([{'qid': '306'}]))), 1000)
        self.assertEqual(q(pd.DataFrame([{'qid': '306'}])).iloc[0].to_dict()['docno'], 'LA021790-0114')
        self.assertEqual(q(pd.DataFrame([{'qid': '306'}])).iloc[0].to_dict()['qid'], '306')
        self.assertEqual(q(pd.DataFrame([{'qid': '306'}])).iloc[999].to_dict()['docno'], 'FBIS4-47956')
        self.assertEqual(q(pd.DataFrame([{'qid': '306'}])).iloc[999].to_dict()['qid'], '306')

    def test_retrieval_submission_from_rest_api_different_id(self):
        from tira.rest_api_client import Client
        from tira.third_party_integrations import ensure_pyterrier_is_loaded
        import pyterrier as pt
        ensure_pyterrier_is_loaded()
        tira = Client()

        q = tira.pt.from_submission('ir-benchmarks/tira-ir-starter/BM25 Re-Rank (tira-ir-starter-pyterrier)', pt.get_dataset("irds:ir-benchmarks/disks45-nocr-trec-robust-2004-20230209-training"))
        self.assertEqual(len(q(pd.DataFrame([{'qid': '306'}]))), 1000)
        self.assertEqual(q(pd.DataFrame([{'qid': '306'}])).iloc[0].to_dict()['docno'], 'LA021790-0114')
        self.assertEqual(q(pd.DataFrame([{'qid': '306'}])).iloc[0].to_dict()['qid'], '306')
        self.assertEqual(q(pd.DataFrame([{'qid': '306'}])).iloc[999].to_dict()['docno'], 'FBIS4-47956')
        self.assertEqual(q(pd.DataFrame([{'qid': '306'}])).iloc[999].to_dict()['qid'], '306')


from typing import List, Tuple
PANDAS_DTYPES = {'docno': str, 'doc_id': str, 'id': str, 'qid': str, 'query_id': str, 'queryid': str, 'docno': str, 'doc_id': str, 'docid': str}

class PandasIntegration():
    """Handling of inputs/outputs in TIRA with pandas. All methods here work in the TIRA sandbox without internet connection (when the data is mounted read-only).
    """
    def __init__(self, tira_client):
        self.tira_client = tira_client

    def from_retriever_submission(self, approach: str, dataset: str, previous_stage: str=None, datasets: List[str]=None):
        """Load a run file as pandas dataframe from tira. Compatible with PyTerrier.

        Args:
            approach (str): the approach for which the run should be loaded, in the format 'task/team/software'.
            dataset (str): the dataset id, either an tira or ir_datasets id.
            previous_stage (str, optional): The previous stage, in case the approach itself is ambigious. Defaults to None.
            datasets (List[str], optional): _description_. Defaults to None.

        Returns:
            pd.DataFrame: The run file parsed to a pandas DataFrame.
        """
        import pandas as pd
        from tira.ir_datasets_util import translate_irds_id_to_tirex
        task, team, software = approach.split('/')

        if dataset and datasets:
            raise ValueError(f'You can not pass both, dataset and datasets. Got dataset = {dataset} and datasets= {datasets}')

        if not datasets:
            datasets = [dataset]

        df_ret = []
        for dataset in datasets:
            ret, run_id = self.tira_client.download_run(task, translate_irds_id_to_tirex(dataset), software, team, previous_stage, return_metadata=True)
            ret['qid'] = ret['query'].astype(str)
            ret['docno'] = ret['docid'].astype(str)
            del ret['query']
            del ret['docid']

            ret['tira_task'] = task
            ret['tira_dataset'] = dataset
            ret['tira_first_stage_run_id'] = run_id
            df_ret += [ret]

        return pd.concat(df_ret)

    def __matching_files(self, approach, dataset, file_selection):
        from glob import glob
        from tira.ir_datasets_util import translate_irds_id_to_tirex
        ret = set()
        
        if type(file_selection) is str:
            file_selection = [file_selection]
        
        for glob_entry in file_selection:
            glob_entry = self.tira_client.get_run_output(approach, (translate_irds_id_to_tirex(dataset))) + glob_entry
            for i in glob(glob_entry): ret.add(i)
        
        return sorted(list(ret))

    def transform_queries(self, approach:str, dataset:str, file_selection: Tuple[str,...]=('/*.jsonl', '/*.jsonl.gz')):
        """Load and transform the query processing outputs specified by the approach on the dataset for direct re-use as a PyTerrier query transformation.

        Args:
            approach str: the approach for which the run should be loaded, in the format 'task/team/software'.
            dataset (str):the dataset id, either an tira or ir_datasets id.
            file_selection (Tuple[str,...], optional): The search glob to outputs specified by the approach on the dataset. Defaults to ('/*.jsonl', '/*.jsonl.gz').

        Raises:
            ValueError: If no approach with the identifier 'approach' was found or if there was an error parsing the outputs.

        Returns:
            pd.DataFrame: a DataFrame with the parsed query processing outputs compatible with PyTerrier query transformations.
        """
        import pandas as pd
        matching_files = self.__matching_files(approach, dataset, file_selection)
        
        if len(matching_files) == 0:
            raise ValueError(f'Could not find a matching query output. Found: {matching_files}. Please specify the file_selection to resolve this.')

        ret = pd.read_json(matching_files[0], lines=True, dtype={'qid': str, 'query': str, 'query_id': str})
        if 'qid' not in ret and 'query_id' in ret:
            ret['qid'] = ret['query_id']
            del ret['query_id']

        return ret

    def transform_documents(self, approach, dataset, file_selection=('/*.jsonl', '/*.jsonl.gz')):
        """Load and transform the document processing outputs specified by the approach on the dataset for direct re-use as a PyTerrier document transformation.

        Args:
            approach str: the approach for which the run should be loaded, in the format 'task/team/software'.
            dataset (str):the dataset id, either an tira or ir_datasets id.
            file_selection (Tuple[str,...], optional): The search glob to outputs specified by the approach on the dataset. Defaults to ('/*.jsonl', '/*.jsonl.gz').

        Raises:
            ValueError: If no approach with the identifier 'approach' was found or if there was an error parsing the outputs.

        Returns:
            pd.DataFrame: a DataFrame with the parsed document processing outputs compatible with PyTerrier document transformations.
        """
        import pandas as pd
        matching_files = self.__matching_files(approach, dataset, file_selection)
        if len(matching_files) == 0:
            raise ValueError('Could not find a matching document output. Used file_selection: ' + str(file_selection) + '. Please specify the file_selection to resolve this.')

        ret = pd.read_json(matching_files[0], lines=True, dtype={'docno': str, 'doc_id': str})

        if 'doc_id' in ret.columns and 'docno' not in ret.columns:
            ret['docno'] = ret['doc_id']
            del ret['doc_id']
        return ret

    def __matching_dataset_files(self, task, dataset, truth_dataset, file_selection):
        from glob import glob
        ret = []
        local_dir = self.tira_client.download_dataset(task, dataset, truth_dataset)

        for glob_entry in file_selection:
            for i in glob(local_dir + glob_entry):
                ret += [i]

        return ret

    def inputs(self, task, dataset, file_selection=('/*.jsonl', '/*.jsonl.gz'), dtype=PANDAS_DTYPES):
        """Load the inputs to systems for a task from tira.

        Args:
            approach str: the approach for which the run should be loaded, in the format 'task/team/software'.
            dataset (str):the dataset id, either an tira or ir_datasets id.
            file_selection (Tuple[str,...], optional): The search glob to outputs specified by the approach on the dataset. Defaults to ('/*.jsonl', '/*.jsonl.gz').
            dtype (Tuple[str,...], optional): Transformations of the data types while loading with pandas. Defaults to PANDAS_DTYPES.

        Raises:
            ValueError: If the dataset is not public or does not exist.

        Returns:
            pd.DataFrame: A DataFrame with all inputs to systems.
        """
        import pandas as pd
        matching_files = self.__matching_dataset_files(task, dataset, False, file_selection)

        if len(matching_files) == 0:
            raise ValueError('Could not find a dataset output. Used file_selection: ' + str(file_selection) + '. Please specify the file_selection to resolve this.')

        return pd.read_json(matching_files[0], lines=True, dtype=dtype)

    def truths(self, task, dataset, file_selection=('/*.jsonl', '/*.jsonl.gz'), dtype=PANDAS_DTYPES):
        """Load the truths, i.e., ground truth labels, for a task from tira.

        Args:
            approach str: the approach for which the run should be loaded, in the format 'task/team/software'.
            dataset (str):the dataset id, either an tira or ir_datasets id.
            file_selection (Tuple[str,...], optional): The search glob to outputs specified by the approach on the dataset. Defaults to ('/*.jsonl', '/*.jsonl.gz').
            dtype (Tuple[str,...], optional): Transformations of the data types while loading with pandas. Defaults to PANDAS_DTYPES.

        Raises:
            ValueError: If the truth is not public or does not exist.

        Returns:
            pd.DataFrame: A DataFrame with all the ground truth labels.
        """
        import pandas as pd
        matching_files = self.__matching_dataset_files(task, dataset, True, file_selection)

        if len(matching_files) == 0:
            raise ValueError('Could not find a dataset output. Used file_selection: ' + str(file_selection) + '. Please specify the file_selection to resolve this.')

        return pd.read_json(matching_files[0], lines=True, dtype=dtype)



import requests
from hashlib import md5
from tira.tira_redirects import redirects, mirror_url
from tira.rest_api_client import Client

def md5_of_first_kilobyte_of_http_resource(url):
    #if not url.startswith('https://files.webis.de'):
    #    raise ValueError(f'URL {url} is not from webis.de respectively zenodo')
    print(url, flush=True)
    return md5(requests.get(url, headers={'Range': f'bytes=0-1024'}).content).hexdigest()


def digest_of_resource(resource):
    from tira.tira_redirects import RESOURCE_REDIRECTS

    if resource not in RESOURCE_REDIRECTS:
        raise ValueError('Could not find resource {resource}.')
    
    url = RESOURCE_REDIRECTS[resource]
    
    return {
        'url': url,
        'redirect_url': redirects(url=url),
        'md5_of_first_kilobyte': md5_of_first_kilobyte_of_http_resource(url)
    }

def digest_of_dataset(dataset_id, truth=False):
    tira_url = f'https://www.tira.io/data-download/training/input-/{dataset_id}.zip'
    url_redirects = redirects(url=tira_url)

    return {
        'tira_url': tira_url,
        'redirect_url': url_redirects,
        'md5_of_first_kilobyte': md5_of_first_kilobyte_of_http_resource(url_redirects['urls'][0])
    }

def digest_of_run_output(approach, dataset_id, run_ids):
    tira = Client()
    task, team, system = approach.split('/')
    run_url = f'https://www.tira.io/task/{task}/user/{team}/dataset/{dataset_id}/download/{run_ids[task][team][system][dataset_id]}.zip'

    return {
        'redirect_url': redirects(approach, dataset_id),
        'run_execution': tira.get_run_execution_or_none(approach, dataset_id),
        'run_url': run_url,
        'redirected_run_url': redirects(url=run_url),
        'md5_of_first_kilobyte': md5_of_first_kilobyte_of_http_resource(redirects(url=run_url)['urls'][0])
    }



#!/usr/bin/env python
import argparse
from tira.local_client import Client
from tira.rest_api_client import Client as RestClient
from tira.local_execution_integration import LocalExecutionIntegration
import os
import shutil
import logging
import tempfile
from tira.third_party_integrations import extract_previous_stages_from_docker_image

from typing import TYPE_CHECKING
if TYPE_CHECKING:
    from .tira_client import TiraClient


def setup_upload_run_command(parser: argparse.ArgumentParser) -> None:
    parser.add_argument('runfile', type=argparse.FileType('rb'), help="Path to the runfile to be uploaded")
    parser.add_argument('--dataset-id', required=True, type=str)
    parser.add_argument('--task-id', required=False, default=os.environ.get('TIRA_TASK_ID'), type=str)
    parser.add_argument('--vm-id', required=False, default=os.environ.get('TIRA_VM_ID'), type=str)
    parser.add_argument('--upload-group', required=True, type=str)

    # Register the upload_run_command method as the "main"-method
    parser.set_defaults(executable=upload_run_command)

def upload_run_command(args: argparse.Namespace) -> int:
    # TODO: allow the user to override some settings (e.g., API key and URL) via arguments
    tira = RestClient()
    if args.task_id is None:
        raise argparse.ArgumentTypeError("Please populate --task-id or set the environment variable TIRA_TASK_ID")
    if args.vm_id is None:
        raise argparse.ArgumentTypeError("Please populate --vm-id or set the environment variable TIRA_VM_ID")
    upload_id = tira.create_upload_group(args.task_id, args.vm_id, args.upload_group)
    success = tira.upload_run(task_id=args.task_id, vm_id=args.vm_id, dataset_id=args.dataset_id, upload_id=upload_id, filestream=args.runfile)
    return 0 if success else 1


def parse_args():
    parser = argparse.ArgumentParser(prog='tira-run')
    parser.add_argument('--input-directory', required=False, default=str(os.path.abspath(".")))
    parser.add_argument('--input-dataset', required=False, default=None)
    parser.add_argument('--input-run', required=False, default=None)
    parser.add_argument('--input-run-directory', required=False, default=None)
    # Not required if the subcommand "tira-run submit" is used. FIXME: the arguments in this group should probably be subcommands
    group = parser.add_mutually_exclusive_group(required=False)
    group.add_argument('--image')
    group.add_argument('--approach')
    group.add_argument('--export-dataset', required=False, default=None, type=str)
    group.add_argument('--export-approach-output', nargs='*', required=False, default=None, type=str)
    group.add_argument('--export-submission-from-jupyter-notebook', required=False, default=None, type=str)
    group.add_argument('--export-submission-environment', nargs='*', required=False, default=None, type=str)
    parser.add_argument('--command', required=False)
    parser.add_argument('--verbose', required=False, default=False, action='store_true')
    parser.add_argument('--evaluate', required=False, default=False, type=bool)
    parser.add_argument('--evaluation-directory', required=False, default=str(os.path.abspath("tira-evaluation")))
    parser.add_argument('--dry-run', required=False, default=False, type=bool)
    parser.add_argument('--dataset-for-export-approach-output', required=False, default=False, type=str)
    parser.add_argument('-v', action='append', default=None, help='Additional volume mounts as <host-path>:<container-path>:<read-write-mode>', required=False)
    parser.add_argument('--allow-network', required=False, default=False, type=bool, help='Is the network available during the execution? (Default = False to improve reproducibility, containers that import ir-datasets can have access to the network, they can be tested with setting allow-network to true.)')
    parser.add_argument('--output-directory', required=False, default=str(os.path.abspath("tira-output")))

    parser.add_argument('--push', required=False, default='false')
    parser.add_argument('--tira-docker-registry-token', required=False, default=os.environ.get('TIRA_DOCKER_REGISTRY_TOKEN'))
    parser.add_argument('--tira-docker-registry-user', required=False, default=os.environ.get('TIRA_DOCKER_REGISTRY_USER'))
    parser.add_argument('--tira-client-token', required=False, default=os.environ.get('TIRA_CLIENT_TOKEN'))
    parser.add_argument('--tira-client-user', required=False, default=os.environ.get('TIRA_CLIENT_USER'))
    parser.add_argument('--tira-vm-id', required=False, default=os.environ.get('TIRA_VM_ID'))
    parser.add_argument('--tira-task-id', required=False, default=os.environ.get('TIRA_TASK_ID'))
    parser.add_argument('--tira-code-repository-id', required=False, default=os.environ.get('TIRA_CODE_REPOSITORY_ID'))
    parser.add_argument('--fail-if-output-is-empty', required=False, default=False, action='store_true', help='After the execution of the software, fail if the output directory is empty.')

    subparsers = parser.add_subparsers()
    setup_upload_run_command(subparsers.add_parser('upload', help="For uploading runs"))

    args = parser.parse_args()
    if args.export_submission_from_jupyter_notebook:
        return args
    
    if args.export_submission_environment:
        return args

    if os.path.exists(args.output_directory) and len(os.listdir(args.output_directory)) > 0:
        parser.error(f'The output directory {os.path.abspath(args.output_directory)} is not empty. Please empty it or provide a new output directory with --ouptut-directory.')
        


    if (args.image is None) == (args.approach is None) == (args.export_dataset):
        parser.error('You have to exclusively use either --approach or --image.')
    if (args.image is None) != (args.command is None):
        args.command = LocalExecutionIntegration().extract_entrypoint(args.image)

        if args.command is not None:
            print(f'Use command from Docker image "{args.command}".')
        else:
            parser.error('I could not find a command to execute, please either configure the entrypoint of the image or use --command.')
    
    args.previous_stages = [] if not args.input_run else [args.input_run]
    if args.input_run is None and args.input_run_directory is None:
        args.input_run = extract_previous_stages_from_docker_image(args.image, args.command)
        args.command = LocalExecutionIntegration().make_command_absolute(args.image, args.command)
        args.previous_stages = args.input_run
        if args.input_run and len(args.input_run) == 1:
            args.input_run = args.input_run[0]

    if args.push.lower() == 'true':
        print('I will check that the API key and the credentials for the image registry are valid.')
        rest_client = RestClient()
        docker_authenticated = rest_client.local_execution.docker_client_is_authenticated()

        api_key_valid = rest_client.api_key_is_valid()

        if args.tira_client_token:
            if api_key_valid:
                args.tira_client_token = rest_client.api_key
            elif 'TIRA_CLIENT_TOKEN' in os.environ and os.environ.get('TIRA_CLIENT_TOKEN'):
                rest_client = RestClient(api_key=os.environ.get('TIRA_CLIENT_TOKEN'))
                api_key_valid = rest_client.api_key_is_valid()
                if api_key_valid:
                    args.tira_client_token = rest_client.api_key
                else:
                    parser.error('The option --tira-client-token (or environment variable TIRA_CLIENT_TOKEN) is required when --push is active.')
            else:
                rest_client = RestClient(api_key=args.tira_client_token)
                if not rest_client.api_key_is_valid():
                    parser.error('The option --tira-client-token (or environment variable TIRA_CLIENT_TOKEN) is required when --push is active.')

        if not args.tira_task_id and args.input_dataset:
            args.tira_task_id = args.input_dataset.split('/')[0]

        if not args.tira_task_id:
            parser.error('The option --tira-task-id (or environment variable TIRA_TASK_ID) is required when --push is active.')

        if not args.tira_vm_id:
            tmp = rest_client.metadata_for_task(args.tira_task_id)
            if tmp and 'status' in tmp and 0 == tmp['status'] and 'context' in tmp and 'user_vms_for_task' in tmp['context']:
                if len(tmp['context']['user_vms_for_task']) != 1:
                    parser.error(f'You have multiple vms ({tmp["context"]["user_vms_for_task"]}), use option --tira-vm-id to specify the vm.')
                else:
                    args.tira_vm_id = tmp['context']['user_vms_for_task'][0]
            else:
                parser.error('The option --tira-vm-id (or environment variable TIRA_VM_ID) is required when --push is active.')

        if not args.tira_client_user:
            tmp = rest_client.metadata_for_task(args.tira_task_id)
            if tmp and 'status' in tmp and 0 == tmp['status'] and 'context' in tmp and 'user_id' in tmp['context']:
                args.tira_client_user = tmp['context']['user_id']
            else:
                parser.error('The option --tira-client-user (or environment variable TIRA_CLIENT_USER) is required when --push is active.')

        if not docker_authenticated and  not args.tira_docker_registry_token and not rest_client.local_execution.login_docker_client(args.tira_task_id, args.tira_vm_id):
                parser.error('The option --tira-docker-registry-token (or environment variable TIRA_DOCKER_REGISTRY_TOKEN) is required when --push is active.')

        if not docker_authenticated and not args.tira_docker_registry_user and not rest_client.local_execution.login_docker_client(args.tira_task_id, args.tira_vm_id):
            parser.error('The option --tira-docker-registry-user (or environment variable TIRA_DOCKER_REGISTRY_USER) is required when --push is active.')

        args.previous_stages = [rest_client.public_runs(args.tira_task_id, args.input_dataset.split('/')[1], i.split('/')[1], i.split('/')[2]) for i in args.previous_stages]

    return args


def main(args=None):
    args = args if args else parse_args()
    logging.basicConfig(level=logging.DEBUG if args.verbose else logging.INFO)

    client: "TiraClient" = Client()

    # Subcommands store their executable into args.executable which takes the parsed arguments and returns an integer
    if hasattr(args, 'executable') and args.executable is not None:
        exit(args.executable(args))

    if args.export_submission_from_jupyter_notebook:
        ret = LocalExecutionIntegration().export_submission_from_jupyter_notebook(args.export_submission_from_jupyter_notebook)
        if not ret:
            exit(1)
            return
        print(ret)
        return

    if args.export_submission_environment:
        from tira.io_utils import all_environment_variables_for_github_action_or_fail
        for i in all_environment_variables_for_github_action_or_fail(args.export_submission_environment):
            print(i)
        return

    if args.export_dataset:
        if len(args.export_dataset.split('/')) != 2:
            print(f'Please pass arguments as: --export-dataset <task>/<tira-dataset>. Got {args.export_dataset}')
            return

        task, dataset = args.export_dataset.split('/')
        print(f'Export dataset "{dataset}" for task "{task}" to directory "{args.output_directory}".')

        tira = RestClient()
        data_dir = tira.download_dataset(task, dataset)

        if os.path.exists(args.output_directory):
            print(f'The directory {args.output_directory} specified by --output-directory already exists. I do not overwrite the directory, please remove it manually if you want to export the dataset.')
        shutil.copytree(data_dir, args.output_directory)
        return

    if args.export_approach_output:
        if not args.dataset_for_export_approach_output:
            print('Please specify the dataset for which the approach outputs should be exported via --dataset-for-export-approach-output')
            return

        if os.path.exists(args.output_directory):
            print(f'The directory {args.output_directory} specified by --output-directory already exists. I do not overwrite the directory, please remove it manually if you want to export the dataset.')
            return

        tira = RestClient()
        data_dirs = []

        for approach in args.export_approach_output:
            if len(approach.split('/')) != 3:
                print(f'Please pass arguments as: --export-approach-output <task>/<tira-dataset>/approach. Got {approach}')

        for approach in args.export_approach_output:
            data_dirs += [tira.get_run_output(approach, args.dataset_for_export_approach_output, True)]

        for num, data_dir in zip(range(len(data_dirs)), data_dirs):
            shutil.copytree(data_dir, Path(args.output_directory) / str(num+1))

        return

    input_dir = args.input_directory
    evaluate = False
    if args.input_dataset and 'none' != args.input_dataset.lower():
        if len(args.input_dataset.split('/')) != 2:
            print(f'Please pass arguments as: --input-dataset <task>/<tira-dataset>. Got {args.input_dataset}')
            return

        task, dataset = args.input_dataset.split('/')
        tira = RestClient()
        print(f'Ensure that the input dataset {dataset} is available.')
        input_dir = tira.download_dataset(task, dataset)
        print(f'Done: Dataset {dataset} is available locally.')

        if args.input_run and type(args.input_run) != list and 'none' != args.input_run.lower():
            print(f'Ensure that the input run {args.input_run} is available.')
            args.input_run = tira.get_run_output(args.input_run, dataset, True)
            print('Done: input run is available locally.')
        if args.input_run and type(args.input_run) == list and len(args.input_run) > 0:
            temp_dir = '/tmp/' + tempfile.TemporaryDirectory().name
            os.makedirs(temp_dir, exist_ok=True)
            for num, input_run in zip(range(len(args.input_run)), args.input_run):
                print(f'Ensure that the input run {input_run} is available.')
                input_run = tira.get_run_output(input_run, dataset, True)
                shutil.copytree(input_run, temp_dir + '/' + str(1+ num))
            args.input_run = temp_dir
        if args.input_run_directory and 'none' != args.input_run_directory.lower():
            args.input_run = os.path.abspath(args.input_run_directory)

        if args.evaluate:
            print(f'Ensure that the evaluation truth for dataset {dataset} is available.')
            evaluate = tira.get_configuration_of_evaluation(task, dataset)
            evaluate['truth_directory'] = tira.download_dataset(task, dataset, truth_dataset=True)
            print(f'Done: Evaluation truth for dataset {dataset} is available.')

    print(f'''
########################################### TIRA RUN CONFIGURATION #########################################################
# image={args.image}
# command={args.command}
############################################################################################################################
''')
    client.local_execution.run(identifier=args.approach, image=args.image, command=args.command, input_dir=input_dir, output_dir=args.output_directory, dry_run=args.dry_run, allow_network=args.allow_network, input_run=args.input_run, additional_volumes=args.v, evaluate=evaluate, eval_dir=args.evaluation_directory)

    if args.fail_if_output_is_empty and (not os.path.exists(args.output_directory) or not os.listdir(args.output_directory)):
        raise ValueError('The software produced an empty output directory, it likely failed?')

    if args.push.lower() == 'true':
        print(f'''
########################################### Review the Outputs of your Software ############################################
# Your software produced the following outputs in {args.output_directory}.
# Please shortly review them before we push your software.
############################################################################################################################
''')
        if not os.path.exists(args.output_directory) or not os.listdir(args.output_directory):
            print('Your software did not produce any output, please review the logs above.')
        else:
            for f in os.listdir(args.output_directory):
                print(f' - {f}: {os.path.getsize(args.output_directory + "/" + f)} bytes')
    
        if not args.fail_if_output_is_empty:
            continue_user = input("Are the outputs correct and should I push the software to TIRA? (y/N) ").lower()

            if continue_user and continue_user.lower() not in ['y', 'yes']:
                print('You did not specify yes, I will not push the software.')
                return

        registry_prefix = client.docker_registry() + '/code-research/tira/tira-user-' + args.tira_vm_id + '/'
        print('Push Docker image')

        image = client.local_execution.push_image(args.image, registry_prefix, args.tira_task_id, args.tira_vm_id)
        print('Upload TIRA_SOFTWARE')
        prev_stages = []
        for i in args.previous_stages:
            prev_stages += [str(i['job_id']['software_id'] if i['job_id']['software_id'] else i['job_id']['upload_id'])]

        tira = RestClient(api_key=args.tira_client_token, api_user_name=args.tira_client_user)
        tira.add_docker_software(image, args.command, args.tira_vm_id, args.tira_task_id, args.tira_code_repository_id, dict(os.environ), prev_stages)



from contextlib import contextmanager
from datetime import datetime
import importlib.util
import json
import logging
import os
import sys
from typing import List, Dict, Callable, Union
from http import HTTPStatus
from http.server import HTTPServer, BaseHTTPRequestHandler
from urllib.parse import urlparse, parse_qs


######################################
# START OF: HTTP response server #####
######################################

_STATUS_OK = {'Status': 'OK'}
_STATUS_INTERNAL_ERROR = {'Status': 'Internal error'}
_STATUS_BAD_REQUEST = {'Status': 'Bad request'}

_predict = None


def _set_predict_function(predict: Callable):
    global _predict
    _predict = predict


def _handle_input(input_list: List) -> Dict:
    if _predict is None or not callable(_predict):
        return _STATUS_INTERNAL_ERROR

    if len(input_list) > 0:
        try:
            result = _predict(input_list)
        except RuntimeError as e:
            logging.error(f'Exception during handling of input \'{input_list}\':\n' + str(e))
            result = None
        if result is None or not isinstance(result, list):
            response = _STATUS_INTERNAL_ERROR
        else:
            response = _STATUS_OK
            response['output'] = result
    else:
        response = _STATUS_BAD_REQUEST

    return response


class InferenceServer(BaseHTTPRequestHandler):

    def _set_headers(self):
        self.send_response(HTTPStatus.OK)
        self.send_header('Content-Type', 'application/json')
        self.send_header('Access-Control-Allow-Origin', '*')  # for local development
        self.end_headers()

    def do_GET(self):
        logging.info(self.command)

        query = parse_qs(urlparse(self.path).query)
        payload_list = query.get('payload', None)

        input_list = None
        response = None

        if payload_list is not None:
            try:
                input_list = [json.loads(element) for element in payload_list]
            except json.JSONDecodeError:
                response = _STATUS_BAD_REQUEST

        if response is None and input_list is not None:
            response = _handle_input(input_list)

        self._set_headers()
        response_string = (json.dumps(response) + '\n').encode('utf-8')
        self.wfile.write(response_string)

    def do_POST(self):
        logging.info(self.command)

        length = int(self.headers.get('content-length'))
        payload = self.rfile.read(length)
        payload_string = payload.decode('utf-8', errors='ignore')

        # ensure list format
        if not payload_string.startswith('[') or not payload_string.endswith(']'):
            payload_string = '[' + payload_string + ']'

        input_list = None
        response = None
        try:
            input_list = json.loads(payload_string)
        except json.JSONDecodeError:
            response = _STATUS_BAD_REQUEST

        if response is None and input_list is not None:
            response = _handle_input(input_list)

        self._set_headers()
        response_string = (json.dumps(response) + '\n').encode('utf-8')
        self.wfile.write(response_string)

    def do_OPTIONS(self):
        self.send_response(HTTPStatus.NO_CONTENT)
        # Send allow-origin header for preflight POST XHRs.
        self.send_header('Access-Control-Allow-Origin', '*')
        self.send_header('Access-Control-Allow-Methods', 'GET, POST')
        self.send_header('Access-Control-Allow-Headers', 'content-type')
        self.end_headers()

    # silence console log messages
    def log_message(self, format, *args):
        pass

####################################
# END OF: HTTP response server #####
####################################


def run_inference_server(base_module: str, absolute_path: str, internal_port: int = 8001, loglevel: int = logging.INFO):
    # logging
    log_filename = 'inference_server_' + datetime.now().strftime('%Y-%m-%d_%H-%M-%S') + '.log'
    _setup_logging(log_filename=log_filename, loglevel=loglevel)

    # load module and import predict function
    with _add_to_path(os.path.dirname(absolute_path)):
        predict = _load_predict_from_imported_module(module_name=base_module, absolute_path=absolute_path)
    if predict is None:
        sys.exit('unable to import predict predict function. See log file for details.')
    _set_predict_function(predict=predict)

    # start HTTP server
    server_address = ('', internal_port)
    httpd = HTTPServer(server_address, InferenceServer)
    logging.info('serving at %s:%d' % (
        len(server_address[0]) > 0 and server_address[0] or 'localhost',
        server_address[1]
    ))
    # console output for safety
    print('[%s] serving at %s:%d' % (
        datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
        len(server_address[0]) > 0 and server_address[0] or 'localhost',
        server_address[1]
    ))
    try:
        httpd.serve_forever()
    except KeyboardInterrupt:
        pass


def _setup_logging(log_filename: str, loglevel: int = logging.INFO):
    log_folder = 'logs'

    if not os.path.isdir(log_folder):
        os.makedirs(log_folder)

    log_file = os.path.join(log_folder, log_filename)

    logging.basicConfig(
        format='[%(asctime)s] %(levelname)s %(message)s',
        filename=log_file,
        level=loglevel,
        datefmt='%Y-%m-%d %H:%M:%S'
    )


def _load_predict_from_imported_module(module_name: str, absolute_path: str = None) -> Union[Callable, None]:
    if module_name in sys.modules:
        logging.warning(f"{module_name!r} already in sys.modules")
        module = importlib.import_module(module_name)
        try:
            predict = getattr(module, 'predict')
        except AttributeError:
            logging.error(f"No function 'predict' found in {module_name!r}")
            return None
    elif absolute_path is not None:
        spec = importlib.util.spec_from_file_location(module_name, absolute_path)
        if spec is not None:
            module = importlib.util.module_from_spec(spec)
            sys.modules[module_name] = module
            spec.loader.exec_module(module)
            logging.info(f"{module_name!r} has been imported")
            try:
                predict = getattr(module, 'predict')
            except AttributeError:
                logging.error(f"No function 'predict' found in {module_name!r}")
                return None
        else:
            logging.error(f"can't find the {module_name!r} module")
            return None
    else:
        logging.error(f"can't find the {module_name!r} module")
        return None

    return predict


@contextmanager
def _add_to_path(p):
    old_path = sys.path.copy()
    try:
        sys.path.insert(0, p)
        yield
    finally:
        sys.path = old_path



import os
from pathlib import Path


class PyTerrierIntegration():
    def __init__(self, tira_client):
        self.tira_client = tira_client
        self.pd = tira_client.pd
        self.irds_docker_image = 'webis/tira-application:0.0.36'

    def retriever(self, approach, dataset=None):
        from tira.pyterrier_util import TiraFullRankTransformer
        input_dir = self.ensure_dataset_is_cached(dataset, dataset)
        return TiraFullRankTransformer(approach, self.tira_client, input_dir)

    def ensure_dataset_is_cached(self, irds_dataset_id, dataset):
        import os
        from pathlib import Path
        from tira.io_utils import run_cmd
        
        cache_dir = self.tira_client.tira_cache_dir + '/pyterrier/' + irds_dataset_id
        full_rank_data = cache_dir + '/full-rank/'
        truth_data = cache_dir + '/truth-data/'
        irds_cache = cache_dir + '/irds-cache/'

        if os.path.isfile(full_rank_data + '/documents.jsonl'):
            return full_rank_data

        Path(full_rank_data).mkdir(parents=True, exist_ok=True)
        Path(truth_data).mkdir(parents=True, exist_ok=True)
        Path(irds_cache).mkdir(parents=True, exist_ok=True)

        run_cmd(['docker', 'run',
                 '-v', irds_cache + ':/root/.ir_datasets/:rw',
                 '-v', full_rank_data + ':/output/:rw',
                 '-v', truth_data + ':/truth/:rw',
                 '--entrypoint', '/irds_cli.sh',
                 self.irds_docker_image,
                 '--output_dataset_path', '/output',
                 '--ir_datasets_id', irds_dataset_id,
                 '--output_dataset_truth_path', '/truth'
                 ])

        return full_rank_data      

    def create_rerank_file(self, run_df=None, run_file=None, irds_dataset_id=None):
        from pathlib import Path
        import tempfile
        from tira.io_utils import run_cmd
        import gzip
        import json
        from tira.third_party_integrations import persist_and_normalize_run

        if run_df is None and run_file is None:
            raise ValueError('Please pass either run_df or run_file')

        if run_file is not None:
            return run_file

        run_file = tempfile.TemporaryDirectory('-rerank-run').name
        Path(run_file).mkdir(parents=True, exist_ok=True)

        if 'text' not in run_df.columns and 'body' not in run_df.columns:
            if not irds_dataset_id:
                raise ValueError(f'Please pass a irds_dataset_id. Got {irds_dataset_id}.')
            persist_and_normalize_run(run_df, 'system-is-ignored', run_file)

            cache_dir = self.tira_client.tira_cache_dir + '/pyterrier/' + irds_dataset_id
            irds_cache = cache_dir + '/irds-cache/'

            run_cmd(['docker', 'run',
                     '-v', irds_cache + ':/root/.ir_datasets/:rw',
                     '-v', run_file + ':/output/:rw',
                     '--entrypoint', '/irds_cli.sh',
                     self.irds_docker_image,
                     '--output_dataset_path', '/output',
                     '--ir_datasets_id', irds_dataset_id,
                     '--rerank', '/output'
            ])
        else:
            with gzip.open(run_file + '/rerank.jsonl.gz', 'wt') as f:
                for _, i in run_df.iterrows():
                    i = i.to_dict()

                    for k in ['original_query', 'original_document']:
                        if k not in i:
                            i[k] = {}

                    if 'text' not in i and 'body' in i:
                        i['text'] = i['body']

                    if 'text' not in i:
                        raise ValueError(f'I expect a field "text", but only found fields {i.keys()}.')

                    f.write(json.dumps(i) + '\n')

        return run_file

    def index(self, approach, dataset):
        """
        Load an PyTerrier index from TIRA.
        """
        import pyterrier as pt
        from tira.ir_datasets_util import translate_irds_id_to_tirex
        ret = self.tira_client.get_run_output(approach, translate_irds_id_to_tirex(dataset)) + '/index'
        return pt.IndexFactory.of(os.path.abspath(ret))

    def from_submission(self, approach, dataset=None, datasets=None):
        software = self.tira_client.docker_software(approach)
        if software.get('ir_re_ranker', False):
            return self.from_retriever_submission(approach, dataset, datasets=datasets)
        else:
            from tira.pyterrier_util import TiraRerankingTransformer
            return TiraRerankingTransformer(approach, self.tira_client, dataset, datasets)

    def from_retriever_submission(self, approach, dataset, previous_stage=None, datasets=None):
        from tira.pyterrier_util import TiraSourceTransformer
        ret = self.pd.from_retriever_submission(approach, dataset, previous_stage, datasets)
        return TiraSourceTransformer(ret)

    def transform_queries(self, approach, dataset, file_selection=('/*.jsonl', '/*.jsonl.gz'), prefix=''):
        from pyterrier.apply import generic
        ret = self.pd.transform_queries(approach, dataset, file_selection)
        cols = [i for i in ret.columns if i not in ['qid']]
        ret = {str(i['qid']): i for _, i in ret.iterrows()}

        def __transform_df(df):
            df = df.copy()
            for col in cols:
                df[prefix + col] = df['qid'].apply(lambda i: ret[str(i)][col])
            return df

        return generic(__transform_df)

    def transform_documents(self, approach, dataset, file_selection=('/*.jsonl', '/*.jsonl.gz'), prefix=''):
        from pyterrier.apply import generic
        ret = self.pd.transform_documents(approach, dataset, file_selection)
        cols = [i for i in ret.columns if i not in ['docno']]
        ret = {str(i['docno']): i for _, i in ret.iterrows()}

        def __transform_df(df):
            for col in cols:
                df[prefix + col] = df['docno'].apply(lambda i: ret[str(i)][col])
            return df

        return generic(__transform_df)

    def doc_features(self, approach, dataset, file_selection=('/*.jsonl', '/*.jsonl.gz')):
        import numpy as np
        from pyterrier.apply import doc_features
        ret = self.pd.transform_documents(approach, dataset, file_selection)
        cols = [i for i in ret.columns if i not in ['docno']]
        ret = {str(i['docno']): np.array([i[c] for c in cols]) for _, i in ret.iterrows()}

        return doc_features(lambda row: ret[str(row['docno'])])

    def query_features(self, approach, dataset, file_selection=('/*.jsonl', '/*.jsonl.gz')):
        import numpy as np
        from pyterrier.transformer import Transformer
        ret = self.pd.transform_queries(approach, dataset, file_selection)
        cols = [i for i in ret.columns if i not in ['qid']]
        ret = {str(i['qid']): np.array([i[c] for c in cols]) for _, i in ret.iterrows()}

        class ApplyQueryFeatureTransformer(Transformer):
            def __repr__(self):
                return "tira.pt.query_features()"

            def transform(self, inputRes):
                outputRes = inputRes.copy()
                
                outputRes["features"] = outputRes.apply(lambda i: ret[str(i['qid'])], axis=1)
                return outputRes
        
        return ApplyQueryFeatureTransformer()

    def reranker(self, approach, irds_id=None):
        from tira.pyterrier_util import TiraLocalExecutionRerankingTransformer
        return TiraLocalExecutionRerankingTransformer(approach, self.tira_client, irds_id=irds_id)

class PyTerrierAnceIntegration():
    """The pyterrier_ance integration to re-use cached ANCE indices. Wraps https://github.com/terrierteam/pyterrier_ance
    """
    def __init__(self, tira_client):
        self.tira_client = tira_client
    
    def ance_retrieval(self, dataset:str):
        """Load a cached pyterrier_ance.ANCEIndexer submitted as workshop-on-open-web-search/ows/pyterrier-anceindex from tira.

        References (for citation):
            https://arxiv.org/pdf/2007.00808.pdf
            https://github.com/microsoft/ANCE/

        Args:
            dataset (str): the dataset id, either an tira or ir_datasets id.

        Returns:
            pyterrier_ance.ANCERetrieval: the ANCE index.
        """
        from tira.ir_datasets_util import translate_irds_id_to_tirex
        ance_index = Path(self.tira_client.get_run_output('ir-lab-sose-2024/ows/pyterrier-anceindex', translate_irds_id_to_tirex(dataset))) / 'anceindex'
        ance_checkpoint = self.tira_client.load_resource('Passage_ANCE_FirstP_Checkpoint.zip')
        import pyterrier_ance
        return pyterrier_ance.ANCERetrieval(ance_checkpoint, ance_index)


class PyTerrierSpladeIntegration():
    """The pyt_splade integration to re-use cached Splade indices. Wraps https://github.com/cmacdonald/pyt_splade
    """
    def __init__(self, tira_client):
        self.tira_client = tira_client

    def splade_index(self, dataset:str, approach: str='workshop-on-open-web-search/naverlabseurope/Splade (Index)'):
        """Load a cached pyt_splade index submitted as the passed approach (default 'workshop-on-open-web-search/naverlabseurope/Splade (Index)') from tira.

        References (for citation):
            https://github.com/naver/splade?tab=readme-ov-file#cite-scroll
            ToDo: Ask Thibault what to cite.

        Args:
            dataset (str): the dataset id, either an tira or ir_datasets id.
            approach (str, optional): the approach id, defaults 'workshop-on-open-web-search/naverlabseurope/Splade (Index)'.

        Returns:
            The PyTerrier index suitable for retrieval.
        """
        from tira.ir_datasets_util import translate_irds_id_to_tirex
        import pyterrier as pt
        ret = Path(self.tira_client.get_run_output('ir-lab-sose-2024/naverlabseurope/Splade (Index)', translate_irds_id_to_tirex(dataset))) / 'spladeindex'
        return pt.IndexFactory.of(os.path.abspath(ret))



from pyterrier.transformer import Transformer, SourceTransformer
import json
import pandas as pd
import tempfile
import gzip
import os


def merge_runs(topics, run_file):
    import numpy as np
    df = pd.read_csv(run_file, sep='\\s+', names=["qid", "q0", "docno", "rank", "score", "system"])
    df['qid'] = df['qid'].astype(str)
    df['docno'] = df['docno'].astype(str)
    topics['qid'] = topics['qid'].astype(str)

    common_columns = np.intersect1d(topics.columns, df.columns)
    join_on = ["qid", "docno"] if 'qid' in common_columns and 'docno' in common_columns else ['qid']

    # we drop columns in topics that exist in the df
    keeping = topics.columns
    drop_columns = [i for i in common_columns if i not in {"qid", "docno"}]
    if len(drop_columns) > 0:
        keeping = topics.columns[~ topics.columns.isin(drop_columns)]

    return topics[keeping].merge(df, how='left', left_on=join_on, right_on=join_on)


class TiraSourceTransformer(SourceTransformer):
    def __init__(self, rtr, **kwargs):
        super().__init__(rtr, **kwargs)

    
    def transform(self, topics):
        import numpy as np
        if 'docno' not in topics.columns:
            return super().transform(topics)
        elif 'qid' not in topics.columns:
            raise ValueError('The dataframe needs to have a column "qid"')
        
        keeping = topics.columns

        common_columns = np.intersect1d(topics.columns, self.df.columns)

        # we drop columns in topics that exist in the self.df
        drop_columns = [i for i in common_columns if i not in ("qid", "docno")]
        if len(drop_columns) > 0:
            keeping = topics.columns[~ topics.columns.isin(drop_columns)]

        return topics[keeping].merge(self.df, on=["qid", "docno"])

class TiraFullRankTransformer(Transformer):
    """
    A Transformer that re-executes some full-rank approach submitted to a shared task in TIRA.
    """

    def __init__(self, approach, tira_client, input_dir, **kwargs):
        self.approach = approach
        self.tira_client = tira_client
        self.input_dir = input_dir

    def transform(self, topics):
        output_dir = tempfile.TemporaryDirectory('-pt-tira-local-execution-full-rank-transformer').name + '/output'
        os.makedirs(output_dir)

        self.tira_client.local_execution.run(identifier=self.approach,
            input_dir=self.input_dir, output_dir=output_dir,
            evaluate=False, dry_run=False
        )

        return merge_runs(topics, output_dir + '/run.txt')


class TiraRerankingTransformer(Transformer):
    """
    A Transformer that loads runs from TIRA that reranked some existing run.
    """

    def __init__(self, approach, tira_client, dataset=None, datasets=None, **kwargs):
        self.task, self.team, self.software = approach.split('/')
        self.tira_client = tira_client
        from tira.ir_datasets_util import translate_irds_id_to_tirex
        if dataset and datasets:
            raise ValueError(f'You can not pass both, dataset and datasets. Got dataset = {dataset} and datasets= {datasets}')

        if not datasets and dataset:
            datasets = [dataset]
        self.datasets = datasets
        
        if self.datasets:
            self.datasets = [translate_irds_id_to_tirex(i) for i in self.datasets]

    def transform(self, topics):
        import numpy as np
        assert "qid" in topics.columns

        if 'tira_task' not in topics.columns or 'tira_dataset' not in topics.columns or 'tira_first_stage_run_id' not in topics.columns:
            if self.datasets:
                tira_configurations = [{'tira_task': self.task, 'tira_dataset': i, 'tira_first_stage_run_id': None} for i in self.datasets]
            else:
                raise ValueError('This run needs to know the tira metadata: tira_task, tira_dataset, and tira_first_stage_run_id needs to be in the columns of the dataframe')
        else:
            tira_configurations = [json.loads(i) for i in topics[['tira_task', 'tira_dataset', 'tira_first_stage_run_id']].apply(lambda i: json.dumps(i.to_dict()), axis=1).unique()]

        df = []
        for tira_configuration in tira_configurations:
            df += [self.tira_client.download_run(tira_configuration['tira_task'], tira_configuration['tira_dataset'], self.software, self.team, tira_configuration['tira_first_stage_run_id'])]
        df = pd.concat(df)
        df['qid'] = df['query'].astype(str)
        df['docno'] = df['docid'].astype(str)
        del df['query']
        del df['docid']

        common_columns = np.intersect1d(topics.columns, df.columns)

        # we drop columns in topics that exist in the df
        keeping = topics.columns
        drop_columns = [i for i in common_columns if i not in {"qid", "docno"}]
        if len(drop_columns) > 0:
            keeping = topics.columns[~ topics.columns.isin(drop_columns)]

        join_criterium = ['qid'] if 'docno' not in keeping else ['qid', 'docno']
        return topics[keeping].merge(df, how='left', left_on=join_criterium, right_on=join_criterium)


class TiraLocalExecutionRerankingTransformer(Transformer):
    """
    A Transformer that re-execues software submitted in TIRA.
    """

    def __init__(self, approach, tira_client, irds_id=None, **kwargs):
        self.task, self.team, self.software = approach.split('/')
        self.approach = approach
        self.tira_client = tira_client
        self.irds_id = irds_id

    def transform(self, topics):
        assert "qid" in topics.columns

        tmp_directory = tempfile.TemporaryDirectory('-pt-tira-local-execution-reranking-transformer').name
        input_dir = self.tira_client.pt.create_rerank_file(run_df=topics, irds_dataset_id=self.irds_id)
        output_dir = tmp_directory + '/output'
        os.makedirs(output_dir)

        self.tira_client.local_execution.run(identifier=self.approach,
            input_dir=input_dir, output_dir=output_dir,
            evaluate=False, dry_run=False
        )

        return merge_runs(topics, tmp_directory + '/output/run.txt')




from tira.rest_api_client import Client
import unittest

# .. todo:: this file still uses "unclean" assertions and should be converted to use unittest assertions

class TestRestAPI(unittest.TestCase):
    tira: Client

    @classmethod
    def setUpClass(cls):
        TestRestAPI.tira = Client()

    def test_all_softwares_works_for_tirex(self):
        actual = TestRestAPI.tira.all_softwares("ir-benchmarks")
        
        assert 'ir-benchmarks/tira-ir-starter/ChatNoir' in actual
        assert 'ir-benchmarks/tira-ir-starter/Index (tira-ir-starter-pyterrier)' in actual

    def test_details_of_public_software_01(self):
        actual = TestRestAPI.tira.docker_software( 'ir-benchmarks/tira-ir-starter/TF_IDF (tira-ir-starter-pyterrier)')

        assert actual is not None
        assert actual['ir_re_ranker'] == False

    def test_output_of_run(self):
        actual = TestRestAPI.tira.get_run_output('ir-benchmarks/tira-ir-starter/BM25 Re-Rank (tira-ir-starter-pyterrier)', 'antique-test-20230107-training')
        actual = open(actual + '/run.txt', 'r').read()
        
        assert actual.startswith('8293 Q0 8293_1 1 33.568841093129976 pyterrier.default_pipelines.wmodel_text_scorer')

    def test_output_of_run_for_redudant_dataset_id(self):
        actual = TestRestAPI.tira.get_run_output('ir-benchmarks/tira-ir-starter/BM25 Re-Rank (tira-ir-starter-pyterrier)', 'ir-benchmarks/antique-test-20230107-training')
        actual = open(actual + '/run.txt', 'r').read()
        
        assert actual.startswith('8293 Q0 8293_1 1 33.568841093129976 pyterrier.default_pipelines.wmodel_text_scorer')

    def test_load_dataset_01(self):
        actual = TestRestAPI.tira.download_dataset('workshop-on-open-web-search', 'retrieval-20231027-training')
        actual = open(actual + '/queries.jsonl', 'r').read()
        
        assert actual.startswith('{"qid": "1", "query": "hubble telescope achievements"}')

    def test_load_dataset_02(self):
        actual = TestRestAPI.tira.download_dataset('workshop-on-open-web-search', 'workshop-on-open-web-search/retrieval-20231027-training')
        actual = open(actual + '/queries.jsonl', 'r').read()
        
        assert actual.startswith('{"qid": "1", "query": "hubble telescope achievements"}')



from tira.local_client import Client

import pandas as pd
import unittest

def query_segmentation(queries):
    queries = pd.DataFrame([{'qid': str(i)} for i in queries])
    tira = Client('tests/resources/')
    query_segmentation =  tira.pt.transform_queries('ir-benchmarks/webis-query-segmentation/wt-snp-baseline', dataset='d1')
    
    ret = query_segmentation(queries)
    return {i['qid']: i['segmentation'] for _, i in ret.iterrows()}

class TestQueryProcessingLoader(unittest.TestCase):
    def test_for_loading_query_segmentations_for_multiple_queries(self):
        expected = {'350': ["health and","computer terminals"], '353': ["antarctica","exploration"]}
        
        actual = query_segmentation([350, 353])
        
        self.assertEqual(len(actual), 2)
        self.assertEqual(actual['350'], expected['350'])
        self.assertEqual(actual['353'], expected['353'])

    def test_for_loading_query_segmentations_for_single_query(self):
        expected = {'351': ["falkland","petroleum exploration"]}
        
        actual = query_segmentation([351])
        
        self.assertEqual(len(actual), 1)
        self.assertEqual(actual['351'], expected['351'])

    def test_pyterrier_can_be_loaded(self):
        from tira.third_party_integrations import ensure_pyterrier_is_loaded
        ensure_pyterrier_is_loaded()

    def test_rest_query_submission_with_rest_api_01(self):
        from tira.rest_api_client import Client
        tira = Client()

        q = tira.pt.transform_queries('ir-benchmarks/ows/query-segmentation-wt-snp', 'disks45-nocr-trec-robust-2004-20230209-training')
        self.assertEqual(len(q(pd.DataFrame([{'qid': '303'}]))), 1)
        self.assertEqual(q(pd.DataFrame([{'qid': '303'}])).iloc[0].to_dict()['segmentation'], ['hubble telescope achievements'])

    def test_rest_query_submission_with_rest_api_02(self):
        from tira.rest_api_client import Client
        tira = Client()

        q = tira.pt.transform_queries('ir-benchmarks/ows/query-segmentation-hyb-a', 'disks45-nocr-trec-robust-2004-20230209-training')
        self.assertEqual(len(q(pd.DataFrame([{'qid': '303'}]))), 1)
        self.assertEqual(q(pd.DataFrame([{'qid': '303'}])).iloc[0].to_dict()['segmentation'], ['hubble telescope', 'achievements'])

    def test_rest_query_submission_with_rest_api_03(self):
        from tira.rest_api_client import Client
        from tira.third_party_integrations import ensure_pyterrier_is_loaded
        import pyterrier as pt
        ensure_pyterrier_is_loaded()
        tira = Client()

        q = tira.pt.transform_queries('ir-benchmarks/fschlatt/query-health-classification', pt.get_dataset("irds:clueweb09/en/trec-web-2009"))
        self.assertEqual(len(q(pd.DataFrame([{'qid': '26'}]))), 1)
        self.assertEqual(q(pd.DataFrame([{'qid': '26'}])).iloc[0].to_dict()['mean_health_score'], 168.416)
        self.assertEqual(q(pd.DataFrame([{'qid': '26'}])).iloc[0].to_dict()['median_health_score'], 130.3137)

    def test_rest_query_submission_with_rest_api_and_custom_selection(self):
        from tira.rest_api_client import Client
        from tira.third_party_integrations import ensure_pyterrier_is_loaded
        import pyterrier as pt
        ensure_pyterrier_is_loaded()
        tira = Client()

        q = tira.pt.transform_queries('ir-benchmarks/fschlatt/query-health-classification', pt.get_dataset("irds:clueweb09/en/trec-web-2009"), '/q*.jsonl')
        self.assertEqual(len(q(pd.DataFrame([{'qid': '26'}]))), 1)
        self.assertEqual(q(pd.DataFrame([{'qid': '26'}])).iloc[0].to_dict()['mean_health_score'], 168.416)
        self.assertEqual(q(pd.DataFrame([{'qid': '26'}])).iloc[0].to_dict()['median_health_score'], 130.3137)

    def test_rest_query_submission_with_rest_api_04(self):
        from tira.rest_api_client import Client
        from tira.third_party_integrations import ensure_pyterrier_is_loaded
        import pyterrier as pt
        ensure_pyterrier_is_loaded()
        tira = Client()

        q = tira.pt.transform_queries('ir-benchmarks/qpptk/fixed-sealer', pt.get_dataset("irds:disks45/nocr/trec-robust-2004"))
        self.assertEqual(len(q(pd.DataFrame([{'qid': '301'}]))), 1)
        self.assertEqual(q(pd.DataFrame([{'qid': '301'}])).iloc[0].to_dict()['max-idf'], 3.5629408815)
        self.assertEqual(q(pd.DataFrame([{'qid': '301'}])).iloc[0].to_dict()['avg-idf'], 2.4740487603)

    def test_rest_query_features(self):
        from tira.rest_api_client import Client
        from tira.third_party_integrations import ensure_pyterrier_is_loaded
        import pyterrier as pt
        ensure_pyterrier_is_loaded()
        tira = Client()

        q = tira.pt.query_features('ir-benchmarks/qpptk/fixed-sealer', pt.get_dataset("irds:disks45/nocr/trec-robust-2004"))
        self.assertEqual(len(q(pd.DataFrame([{'qid': '301'}]))), 1)
        self.assertEqual(q(pd.DataFrame([{'qid': '301'}])).iloc[0].to_dict()['features'][0], 3.5629408815)
        self.assertEqual(q(pd.DataFrame([{'qid': '301'}])).iloc[0].to_dict()['features'][1], 2.4740487603)



import unittest
from tests.test_utils import digest_of_dataset, digest_of_run_output
from approvaltests import verify_as_json

class IrLabSoSe2024Test(unittest.TestCase):
    """Unit tests for all use cases of the ir lab in summer semester 2024.
    """
    def test_corpus_is_available(self):
        actual = digest_of_dataset('ir-acl-anthology-20240411-training')
        verify_as_json(actual)

    
    def test_digest_of_run_outputs(self):
        dataset_id = 'ir-acl-anthology-20240411-training'
        approaches = [
            'ir-lab-sose-2024/tira-ir-starter/Index (tira-ir-starter-pyterrier)',
            'ir-lab-sose-2024/tira-ir-starter/Index (pyterrier-stanford-lemmatizer)',
            #'ir-lab-sose-2024/seanmacavaney/DocT5Query',
            #'ir-lab-sose-2024/seanmacavaney/corpus-graph',
            'ir-lab-sose-2024/ows/pyterrier-anceindex',
            'ir-lab-sose-2024/naverlabseurope/Splade (Index)',
        ]
        actual = {i: {} for i in approaches}

        run_ids = {"ir-lab-sose-2024": {
            "tira-ir-starter": {
                "Index (tira-ir-starter-pyterrier)": {"ir-acl-anthology-20240411-training": "2024-04-11-19-43-23"},
                "Index (pyterrier-stanford-lemmatizer)": {"ir-acl-anthology-20240411-training": "2024-04-16-11-05-06"}
            }, 
            "seanmacavaney": {
                "DocT5Query": {"ir-acl-anthology-20240411-training": ""},
                "corpus-graph": {"ir-acl-anthology-20240411-training": ""}
            },
            "ows": {
                "pyterrier-anceindex": {"ir-acl-anthology-20240411-training": "2024-04-11-19-47-18"}
            },
            "naverlabseurope": {
                "Splade (Index)": {"ir-acl-anthology-20240411-training": "2024-04-14-08-40-58"}
            }
        }}

        for approach in approaches:
            actual[approach][dataset_id] = digest_of_run_output(approach, dataset_id, run_ids)

        verify_as_json(actual)


from tira.third_party_integrations import load_rerank_data
import unittest

# .. todo:: there are no assertions in this file

class TestLoadRerankData(unittest.TestCase):
    def test_loading_rerank_data_from_local_file(self):
        re_rank_data = load_rerank_data(default='tests/resources/re-ranking-outputs/')
        
        assert 3 == len(re_rank_data)
        assert {"qid": "1", "query": "query 1", "docno": "doc-1", "text": "Text of doc-1", "rank": 1, "score": 10} == re_rank_data.iloc[0].to_dict()
        assert {"qid": "3", "query": "query 3", "docno": "doc-3", "text": "Text of doc-3", "rank": 1, "score": 1} == re_rank_data.iloc[2].to_dict()

    def test_loading_rerank_data_from_remote_file(self):
        re_rank_data = load_rerank_data(default='workshop-on-open-web-search/re-ranking-20231027-training')
        
        assert 6 == len(re_rank_data)
        assert {"qid": "2", "query": "how to exit vim?", "docno": "doc-1", "text": "Press ESC key, then the : (colon), and type the wq command after the colon and hit the Enter key to save and leave Vim.", "rank": 1, "score": 10} == re_rank_data.iloc[2].to_dict()



from tira.rest_api_client import Client
import unittest

class TestLocalExecutionIntegrationTest(unittest.TestCase):
    def test_pd_load_inputs_wows_re_ranking(self):
        tira = Client()
        actual = tira.pd.inputs('workshop-on-open-web-search', 're-ranking-20231027-training')
        self.assertEqual(6, len(actual))
        self.assertEqual('hubble telescope achievements', actual.iloc[0]['query'])
        self.assertEqual('The Hubble telescope discovered two moons of Pluto, Nix and Hydra.', actual.iloc[0]['text'])
        self.assertEqual(1, actual.iloc[0]['rank'])
        self.assertEqual('1', actual.iloc[0]['qid'])

    def test_pd_load_truths_wows_re_ranking(self):
        tira = Client()
        actual = tira.pd.truths('workshop-on-open-web-search', 're-ranking-20231027-training')
        self.assertEqual(3, len(actual))
        self.assertEqual('hubble telescope achievements', actual.iloc[0]['query'])
        self.assertEqual('1', actual.iloc[0]['qid'])



import unittest
from approvaltests import verify_as_json
from tests.test_utils import digest_of_resource

class TestLoadResources(unittest.TestCase):
    def test_redirects_in_incubator(self):
        actual = digest_of_resource('Passage_ANCE_FirstP_Checkpoint.zip')
        verify_as_json(actual)

    def test_redirects_in_incubator_for_stanford_lemmatiser(self):
        actual = digest_of_resource('custom-terrier-token-processing-1.0-SNAPSHOT-jar-with-dependencies.jar')
        verify_as_json(actual)



from tira.local_execution_integration import LocalExecutionIntegration as Client
import unittest

class TestLocalExecutionIntegrationTest(unittest.TestCase):
    def test_01(self):
        expected = 'registry.webis.de/code-research/tira/tira-user-dam/seupd2324:latest'
        actual = Client().normalize_image_name('seupd2324', 'registry.webis.de/code-research/tira/tira-user-dam/')

        self.assertEqual(expected, actual)

    def test_02(self):
        expected = 'registry.webis.de/code-research/tira/tira-user-dam/seupd2324:latest'
        actual = Client().normalize_image_name('seupd2324-dam-baseline', 'registry.webis.de/code-research/tira/tira-user-dam/')

        self.assertEqual(expected, actual)

    def test_03(self):
        expected = 'registry.webis.de/code-research/tira/tira-user-petropoulossiblings/panospetr:1.1.2'
        actual = Client().normalize_image_name('panospetr/av-software-petropoulos-pan2024:1.1.2', 'registry.webis.de/code-research/tira/tira-user-petropoulossiblings/')

        self.assertEqual(expected, actual)
